import { MinusCircleOutlined, PlusOutlined } from "@ant-design/icons"
import { Button, Card, Form, Modal, Popconfirm, Space, message } from "antd"
import React, { useEffect, useState } from "react"
import styles from './index.less'
import VideoNews from "@/pages/launchSystemNew/components/newsModal/videoNews"
import { useModel } from "umi"
import SelectCloud from "@/pages/launchSystemNew/components/selectCloud"
import { getVideoImgUrl } from "@/utils/utils"
import VideoFrameSelect from "@/pages/launchSystemV3/components/VideoFrameSelect"

interface Props {
    creativeTemplateId: number
    materialData: any
    value?: any,
    visible?: boolean
    onClose?: () => void
    onChange?: (value: any) => void
}

const AddMaterial: React.FC<Props> = ({ creativeTemplateId, materialData, visible, value, onChange, onClose }) => {

    /*********************************/
    const { init } = useModel('useLaunchAdq.useBdMediaPup')

    const [form] = Form.useForm();
    const dynamicGroup = Form.useWatch('dynamicGroup', form)

    const [materialConfig, setMaterialConfig] = useState<{
        adcreativeTemplateId?: number,
        type: string,
        cloudSize: { relation: string, width: number, height: number }[],
        list: any[],
        index: number,
        max: number,
        sliderImgContent: any
    }>({
        type: '',//类型
        cloudSize: [],//素材搜索条件
        list: [],//素材
        index: 0, // 素材组下标
        max: 1,//素材数量
        sliderImgContent: undefined
    })//图片素材配置
    const [selectVideoVisible, setSelectVideoVisible] = useState(false)
    const [mLength, setMLength] = useState<number>(1)
    const [videoUploads, setVideoUploads] = useState<any>()
    const [imgUploads, setImgUploads] = useState<any>()
    const [minNumber, setMinNumber] = useState<number>(3)
    /*********************************/

    useEffect(() => {
        console.log('materialData--->', materialData)
        if (materialData && Object.keys(materialData).length > 0) {
            let fKey = Object.keys(materialData)[0];
            let children = materialData[fKey]?.children
            let childrenKey = Object.keys(children)
            setMLength(childrenKey?.length)

            let videoData = childrenKey?.find(item => item === 'short_video1' || item === 'video_id')
            if (videoData) {
                setVideoUploads(children[videoData])
            }
            let imageData = childrenKey?.find(item => item === 'cover_id' || item === 'image_id')
            if (imageData) {
                setImgUploads(children[imageData])
            }
            let imageListData = childrenKey?.find(item => item === 'list')
            if (imageListData) {
                let data = children[imageListData]
                if (fKey === 'image_list') {
                    setMinNumber(data.arrayProperty.mixNumber)
                    setImgUploads({ ...data['children']['image_id'], arrayProperty: data.arrayProperty, name: 'image_list' })
                } else if (fKey === 'element_story') {
                    setMinNumber(data.arrayProperty.mixNumber)
                    setImgUploads({ ...data['children']['image'], arrayProperty: data.arrayProperty, name: 'element_story' })
                }
            }
            // setImgUploads(childrenKey?.find(item => item === 'cover_id' || item === 'list' || item === 'image_id'))
        } else {
            setVideoUploads({})
            setImgUploads({})
        }
    }, [materialData])

    const setFrame = (url: string, num: number, field: string) => {
        let newDynamicGroup = dynamicGroup?.map((item: any, index: number) => {
            if (num === index) {
                if (item) {
                    item[field] = { ...item[field], url, id: null }
                    return { ...item }
                } else {
                    return { [field]: { url, id: null, materialType: 0 } }
                }
            }
            return item
        })
        form.setFieldsValue({ dynamicGroup: newDynamicGroup })
    }

    const handleOk = (values: any) => {
        console.log(values)
        onChange?.(values)
    }

    useEffect(() => {
        console.log('value--->', value)
        if (value) {
            form.setFieldsValue({ ...value })
        }
    }, [value])

    return <Modal
        title={<Space>
            <strong style={{ fontSize: 20 }}>创意素材</strong>
            {videoUploads && Object.keys(videoUploads)?.length > 0 && <Button type="link" onClick={() => {
                init({ mediaType: 'VIDEO', num: 100, cloudSize: creativeTemplateId === 1708 ? [[{ relation: '=', width: 1280, height: 720 }]] : [[{ relation: '=', width: videoUploads.restriction.videoRestriction.minWidth, height: videoUploads.restriction.videoRestriction.minHeight }]], maxSize: videoUploads.restriction.videoRestriction.fileSize * 1024 })
                setMaterialConfig({
                    ...materialConfig,
                    type: videoUploads.name,
                    max: 1,
                    index: 99999,
                    adcreativeTemplateId: creativeTemplateId
                })
                setTimeout(() => {
                    setSelectVideoVisible(true)
                }, 100)
            }}>批量添加视频素材</Button>}
            {imgUploads && Object.keys(imgUploads)?.length > 0 && <Button type="link" onClick={() => {
                init({ mediaType: 'IMG', num: 100, cloudSize: [[{ relation: '=', width: imgUploads.restriction.imageRestriction.width, height: imgUploads.restriction.imageRestriction.height }]], maxSize: imgUploads.restriction.imageRestriction.fileSize * 1024 })
                setMaterialConfig({
                    ...materialConfig,
                    type: imgUploads.name,
                    max: (imgUploads.name === 'image_list' || imgUploads.name === 'element_story') ? imgUploads.arrayProperty.maxNumber : 1,
                    index: 99999,
                    adcreativeTemplateId: creativeTemplateId
                })
                setTimeout(() => {
                    setSelectVideoVisible(true)
                }, 100)
            }}>批量添加图片素材</Button>}
            {(dynamicGroup && dynamicGroup?.length > 1) && <Popconfirm
                title="是否清空?"
                onConfirm={() => form.setFieldsValue({ dynamicGroup: [undefined] })}
                okText="是"
                cancelText="否"
            >
                <Button type="link" danger>一键清空</Button>
            </Popconfirm>}
        </Space>}
        visible={visible}
        onCancel={onClose}
        footer={null}
        width={900}
        className={`modalResetCss`}
        bodyStyle={{ padding: '0 0 40px', position: 'relative', borderRadius: '0 0 8px 8px' }}
        maskClosable={false}
    >
        <Form
            form={form}
            name="newMaterial"
            labelAlign='left'
            layout="vertical"
            colon={false}
            style={{ backgroundColor: '#f1f4fc', maxHeight: 650, overflow: 'hidden', overflowY: 'auto', padding: '0 10px 10px', borderRadius: '0 0 8px 8px' }}
            scrollToFirstError={{
                behavior: 'smooth',
                block: 'center'
            }}
            onFinishFailed={({ errorFields }) => {
                message.error(errorFields?.[0]?.errors?.[0])
            }}
            onFinish={handleOk}
            initialValues={{
                dynamicGroup: [undefined]
            }}
        >
            <Form.List name="dynamicGroup">
                {(fields, { add, remove }) => (<>
                    {fields.map((field, num) => (<Card
                        title={<strong style={{ fontSize: 18 }}>创意组{num + 1}</strong>}
                        className="cardResetCss"
                        key={field.key}
                        style={{ marginTop: 10, marginBottom: 10 }}
                    >
                        <Space size={30} style={{ width: '100%' }} className={styles.space}>
                            {Object.keys(materialData)?.map(key => {
                                let m = materialData[key]
                                let children = m.children

                                return Object.keys(children).map(k => {
                                    let item = children[k]
                                    if (k === 'short_video1' || k === 'video_id') {
                                        return <Form.Item
                                            {...field}
                                            label={<strong>{item.description}</strong>}
                                            rules={[{ required: true, message: '请选择素材!' }]}
                                            name={[field.name, item.name]}
                                            key={k}
                                        >
                                            <div className={`${styles.box} ${styles.video}`} style={{ width: 300, height: 160 }} onClick={() => {
                                                init({ mediaType: 'VIDEO', cloudSize: creativeTemplateId === 1708 ? [[{ relation: '=', width: 1280, height: 720 }]] : [[{ relation: '=', width: item.restriction.videoRestriction.minWidth, height: item.restriction.videoRestriction.minHeight }]], maxSize: item.restriction.videoRestriction.fileSize * 1024 })
                                                setMaterialConfig({
                                                    ...materialConfig,
                                                    type: item.name,
                                                    max: 1,
                                                    index: num,
                                                    adcreativeTemplateId: creativeTemplateId
                                                })
                                                setTimeout(() => {
                                                    setSelectVideoVisible(true)
                                                }, 100)
                                            }}>
                                                <div className={styles.p}>
                                                    {dynamicGroup?.length > 0 && dynamicGroup[num] && Object.keys(dynamicGroup[num])?.includes(item.name) ? <VideoNews src={dynamicGroup[num][item.name]['url']} style={{ display: 'block', width: 'auto', margin: 0, height: '100%' }} maskImgStyle={{ position: 'absolute', top: '50%', left: '50%', width: 40, height: 40, transform: 'translate(-50%, -50%)', zIndex: 10 }} /> : <>
                                                        <span>{`推荐尺寸（${creativeTemplateId === 1708 ? 1280 : item.restriction.videoRestriction.minWidth} x ${creativeTemplateId === 1708 ? 720 : item.restriction.videoRestriction.minHeight}）`}</span>
                                                        <span>{`${item.restriction.videoRestriction.fileFormat?.map((str: any) => str?.replace('MEDIA_TYPE_', ''))};< ${item.restriction.videoRestriction.fileSize / 1024}M;时长 ≥ ${item.restriction.videoRestriction.minDuration}s,≤ ${item.restriction.videoRestriction.maxDuration}s,必须带有声音`}</span>
                                                    </>}
                                                </div>
                                            </div>
                                        </Form.Item>
                                    }
                                    if (item.name === 'image_id' || item.name === 'cover_id') {
                                        return <Form.Item
                                            {...field}
                                            label={<strong>{item.description}</strong>}
                                            rules={[{ required: true, message: '请选择素材!' }]}
                                            name={[field.name, item.name]}
                                            key={key}
                                        >
                                            <Space align="end">
                                                <div className={`${styles.box} ${styles.image}`} style={{ width: 300, height: 160 }} onClick={() => {
                                                    init({ mediaType: 'IMG', cloudSize: [[{ relation: '=', width: item.restriction.imageRestriction.width, height: item.restriction.imageRestriction.height }]], maxSize: item.restriction.imageRestriction.fileSize * 1024 })
                                                    setMaterialConfig({
                                                        ...materialConfig,
                                                        type: item.name,
                                                        max: 1,
                                                        index: num,
                                                        adcreativeTemplateId: creativeTemplateId
                                                    })
                                                    setTimeout(() => {
                                                        setSelectVideoVisible(true)
                                                    }, 100)
                                                }}>
                                                    <p>
                                                        {dynamicGroup?.length > 0 && dynamicGroup[num] && Object.keys(dynamicGroup[num])?.includes(item.name) ? <img src={dynamicGroup[num][item.name]['url']} /> : <>
                                                            <span>{`推荐尺寸（${item.restriction.imageRestriction.width} x ${item.restriction.imageRestriction.height}）`}</span>
                                                            <span>{`${item.restriction.imageRestriction.fileFormat?.map((str: any) => str?.replace('IMAGE_TYPE_', ''))}；小于 ${item.restriction.imageRestriction.fileSize}KB`}</span>
                                                        </>}
                                                    </p>
                                                </div>
                                                {videoUploads && Object.keys(videoUploads)?.length > 0 && <div style={{ width: 32 }}>
                                                    {dynamicGroup?.length > 0 && dynamicGroup[num] && (Object.keys(dynamicGroup[num])?.includes('video_id') || Object.keys(dynamicGroup[num])?.includes('short_video1')) && <VideoFrameSelect onChange={(e) => setFrame(e, num, item.name)} url={dynamicGroup[num]?.['video_id']?.['url'] || dynamicGroup[num]?.['short_video1']?.['url']} />}
                                                </div>}
                                            </Space>
                                        </Form.Item>
                                    }

                                    if (item.name === 'list') {
                                        let name = ''
                                        let imageData: any = {}
                                        if (key === 'image_list') {
                                            imageData = item.children.image_id
                                            name = 'image_list';
                                        } else if (key === 'element_story') {
                                            imageData = item.children.image
                                            name = 'element_story'
                                        }
                                        return <Form.Item
                                            {...field}
                                            label={<strong>{imageData.description}</strong>}
                                            rules={[
                                                { required: true, type: 'array', len: minNumber || item.arrayProperty.maxNumber, message: '素材数量不对!' },
                                                { required: true, message: '请选择素材!' },
                                            ]}
                                            name={[field.name, name]}
                                            key={key}
                                        >
                                            <div className={`${styles.box} ${item.arrayProperty.maxNumber >= 3 ? styles.image_list : styles.imageMater}`} style={item.arrayProperty.maxNumber >= 3 ? { flexFlow: 'row', width: '100%', gap: 2 } : {}} onClick={() => {
                                                init({ mediaType: 'IMG', num: item.arrayProperty.maxNumber, cloudSize: [[{ relation: '=', width: imageData.restriction.imageRestriction.width, height: imageData.restriction.imageRestriction.height }]], maxSize: imageData.restriction.imageRestriction.fileSize * 1024 })
                                                setMaterialConfig({
                                                    ...materialConfig,
                                                    type: name,
                                                    max: item.arrayProperty.maxNumber,
                                                    index: num,
                                                    adcreativeTemplateId: creativeTemplateId
                                                })
                                                setTimeout(() => {
                                                    setSelectVideoVisible(true)
                                                }, 100)
                                            }}>
                                                {Array(item.arrayProperty.maxNumber).fill('').map((arr, index1) => {
                                                    return <p key={index1} style={item.arrayProperty.maxNumber >= 3 ? { width: 130, height: 130 } : { justifyContent: 'center' }}>
                                                        {dynamicGroup?.length > 0 && dynamicGroup[num] && Object.keys(dynamicGroup[num])?.includes(name) && dynamicGroup[num][name][index1] ? <img src={dynamicGroup[num][name][index1]['url']} /> : <>
                                                            <span>{`推荐尺寸（${imageData.restriction.imageRestriction.width} x ${imageData.restriction.imageRestriction.height}）`}</span>
                                                            <span>{`${imageData.restriction.imageRestriction.fileFormat?.map((str: any) => str?.replace('IMAGE_TYPE_', ''))}；小于 ${imageData.restriction.imageRestriction.fileSize}KB`}</span>
                                                        </>}
                                                    </p>
                                                })}
                                            </div>
                                        </Form.Item>
                                    }

                                    return null
                                })
                            })}
                            {fields?.length > 1 && <MinusCircleOutlined className={styles.clear} onClick={() => remove(field.name)} style={{ marginBottom: 24, color: 'red' }} />}
                        </Space>
                    </Card>))}

                    <Form.Item style={{ marginBottom: 0 }}>
                        <Button type="dashed" style={{ color: '#1890ff' }} onClick={() => add()} block icon={<PlusOutlined />}>添加创意组</Button>
                    </Form.Item>
                </>)}
            </Form.List>
            <Form.Item className="submit_pull">
                <Space>
                    <Button onClick={onClose}>取消</Button>
                    <Button type="primary" htmlType="submit" className="modalResetCss">
                        确定
                    </Button>
                </Space>
            </Form.Item>
        </Form>


        {/* 选择视频素材 */}
        {selectVideoVisible && <SelectCloud
            visible={selectVideoVisible}
            onClose={() => setSelectVideoVisible(false)}
            sliderImgContent={materialConfig.index === 99999 ? undefined :
                materialConfig.type === 'element_story' ? (dynamicGroup[materialConfig.index] && Object.keys(dynamicGroup[materialConfig.index])?.includes('element_story')) ? dynamicGroup[materialConfig.index]['element_story']?.map((item: any) => item) : undefined :
                    materialConfig.type === 'image_list' ? (dynamicGroup[materialConfig.index] && Object.keys(dynamicGroup[materialConfig.index])?.includes('image_list')) ? dynamicGroup[materialConfig.index]['image_list']?.map((item: any) => item) : undefined :
                        (dynamicGroup[materialConfig.index] && Object.keys(dynamicGroup[materialConfig.index])?.includes(materialConfig.type)) ? [{ url: dynamicGroup[materialConfig.index][materialConfig.type] }] : undefined
            }
            onChange={(content: any) => {
                if (content.length > 0) {
                    if (materialConfig.index === 99999) {
                        if (materialConfig.type === 'image_list' || materialConfig.type === 'element_story') {
                            let urls = content?.map((item: any) => ({ id: item?.id, url: item?.url, materialType: 0 }))
                            let max = materialConfig.max
                            let materialsNew = dynamicGroup.map((item: any) => {
                                let newItem = item || {}
                                // 判断是否有字段，是否设置了值
                                if (Object.keys(newItem).includes(materialConfig.type) && newItem[materialConfig.type]) {
                                    if (max > newItem[materialConfig.type].length && urls.length > 0) {
                                        let difference = max - newItem[materialConfig.type].length
                                        let material: any[] = []
                                        if (urls.length >= difference) {
                                            material = urls.splice(0, difference)
                                        } else {
                                            material = urls.splice(0, urls.length)
                                        }
                                        newItem[materialConfig.type] = [...newItem[materialConfig.type], ...material]
                                        return newItem
                                    } else {
                                        return newItem
                                    }
                                } else {
                                    if (urls.length >= max) {
                                        let material = urls.splice(0, max)
                                        return { ...newItem, [materialConfig.type]: material }
                                    } else if (urls.length > 0) {
                                        let material = urls.splice(0, urls.length)
                                        return { ...newItem, [materialConfig.type]: material }
                                    } else {
                                        return newItem
                                    }
                                }
                            })
                            if (urls.length > 0) {
                                let data = Array(Math.ceil(urls.length / max)).fill(undefined).map(item => {
                                    if (urls.length >= max) {
                                        let material = urls.splice(0, max)
                                        return { [materialConfig.type]: material }
                                    } else {
                                        let material = urls.splice(0, urls.length)
                                        return { [materialConfig.type]: material }
                                    }
                                })
                                materialsNew = [...materialsNew, ...data]
                            }
                            console.log('materialsNew-->', materialsNew)
                            form.setFieldsValue({ dynamicGroup: materialsNew })
                        } else {
                            let newMaterials = content?.map((item: any) => {
                                if (["short_video1", 'video_id'].includes(materialConfig.type) && mLength === 2) {
                                    return { [materialConfig.type]: { url: item?.url, id: item?.id, materialType: 0 }, cover_id: { url: getVideoImgUrl(item?.url), id: null, materialType: 0 } }
                                }
                                return { [materialConfig.type]: { url: item?.url, id: item?.id, materialType: 0 } }
                            })
                            if (newMaterials.length > 0) {
                                if (dynamicGroup?.every((item: any) => !item)) { // 没设置过
                                    form.setFieldsValue({ dynamicGroup: newMaterials })
                                } else { // 设置过
                                    let materialsNew = dynamicGroup.map((item: any) => {
                                        let newItem = item || {}
                                        if (Object.keys(newItem).includes(materialConfig.type) && newItem[materialConfig.type]) {
                                            return item
                                        } else {
                                            if (newMaterials.length > 0) {
                                                let material = newMaterials.splice(0, 1)
                                                return { ...newItem, ...material[0] }
                                            } else {
                                                return item
                                            }
                                        }
                                    })
                                    if (newMaterials.length > 0) {
                                        materialsNew = [...materialsNew, ...newMaterials]
                                    }
                                    form.setFieldsValue({ dynamicGroup: materialsNew })
                                }
                            }
                        }
                    } else {
                        let newDynamicGroup = dynamicGroup?.map((item: any, index: number) => {
                            if (materialConfig.index === index) {
                                if (materialConfig.type === 'image_list' || materialConfig.type === 'element_story') {
                                    if (item) {
                                        item[materialConfig.type] = content?.map((item: any) => ({ id: item?.id, url: item?.url, materialType: 0 }))
                                        return { ...item }
                                    } else {
                                        return { [materialConfig.type]: content?.map((item: any) => ({ id: item?.id, url: item?.url, materialType: 0 })) }
                                    }
                                } else {
                                    if (item) {
                                        item[materialConfig.type] = { id: content[0]?.id, url: content[0]?.url, materialType: 0 }
                                        return { ...item }
                                    } else {
                                        if (["short_video1", 'video_id'].includes(materialConfig.type) && mLength === 2) {
                                            return { [materialConfig.type]: { id: content[0]?.id, url: content[0]?.url, materialType: 0 }, cover_id: { id: null, url: getVideoImgUrl(content[0]?.url), materialType: 0 } }
                                        }
                                        return { [materialConfig.type]: { id: content[0]?.id, url: content[0]?.url, materialType: 0 } }
                                    }
                                }
                            }
                            return item
                        })
                        form.setFieldsValue({ dynamicGroup: newDynamicGroup })
                    }
                }
                setSelectVideoVisible(false)
            }}
        />}
    </Modal>
}

export default React.memo(AddMaterial)