import New1Radio from "@/pages/launchSystemV3/components/New1Radio"
import TextAideInput from "@/pages/launchSystemV3/components/TextAideInput"
import { txtLength } from "@/utils/utils"
import { Button, Card, Form, Modal, Space, message } from "antd"
import React, { useEffect, useState } from "react"


interface Props {
    textData: any,
    dynamicMaterialDTos: any,
    value?: any,
    visible?: boolean
    onClose?: () => void
    onChange?: (value: any) => void
}

/**
 * 创意文案
 * @param param0 
 * @returns 
 */
const NewText: React.FC<Props> = ({ visible, onClose, onChange, value, textData, dynamicMaterialDTos }) => {

    /*************************************/
    const [form] = Form.useForm();
    const type = Form.useWatch('type', form)
    const textDto = Form.useWatch('textDto', form)

    const [textList, setTextList] = useState<PULLIN.TextDtoProps[]>([])
    /*************************************/

    const handleOk = (values: any) => {
        console.log(values)
        const { type, textDto } = values
        onChange?.({
            type,
            dynamicCreativesTextDetailDTOList: textDto.map((item: any) => {
                let data: any = {}
                Object.keys(item).forEach(key => {
                    data[key] = [item[key]]
                })
                return data
            })
        })
    }

    useEffect(() => {
        console.log('创意文案--->', value)
        if (value && Object.keys(value).length) {
            const { type, dynamicCreativesTextDetailDTOList } = value
            form.setFieldsValue({ type, textDto: dynamicCreativesTextDetailDTOList.map((item: any) => {
                let data: any = {}
                Object.keys(item).forEach(key => {
                    data[key] = item[key]?.[0]
                })
                return data
            }) })
        }
    }, [value])

    useEffect(() => {
        if (textData && Object.keys(textData)) {
            let data = Object.values(textData).map((item: any) => {
                let content = item.children.content
                return { label: content.description, restriction: content.restriction, value: item.name, required: item.required }
            })
            setTextList(data)
        }
    }, [textData])

    return <Modal
        title={<strong style={{ fontSize: 20 }}>创意文案</strong>}
        visible={visible}
        onCancel={onClose}
        footer={null}
        width={700}
        className={`modalResetCss`}
        bodyStyle={{ padding: '0 0 40px', position: 'relative', borderRadius: '0 0 8px 8px' }}
        maskClosable={false}
    >
        <Form
            form={form}
            name="newText"
            labelAlign='left'
            labelCol={{ span: 5 }}
            colon={false}
            style={{ backgroundColor: '#f1f4fc', maxHeight: 650, overflow: 'hidden', overflowY: 'auto', padding: '0 10px 10px', borderRadius: '0 0 8px 8px' }}
            scrollToFirstError={{
                behavior: 'smooth',
                block: 'center'
            }}
            onFinishFailed={({ errorFields }) => {
                message.error(errorFields?.[0]?.errors?.[0])
            }}
            initialValues={{
                type: 0,
                textDto: [{}]
            }}
            onFinish={handleOk}
        >
            <Card className="cardResetCss" style={{ marginTop: 10 }}>
                <Form.Item name="type" label={<strong>文案分配规则</strong>} style={{ marginBottom: 0 }} rules={[{ required: true, message: '请选择营销目的!' }]}>
                    <New1Radio
                        data={[{ label: '全部相同', value: 0 }, { label: '按创意顺序分配', value: 1 }]}
                        onChange={(value) => {
                            if (value === 0) {
                                form.setFieldsValue({ textDto: [textDto[0] || {}] })
                            } else if (value === 1) {
                                let oldtextDto: PULLIN.TextDtoProps[] = JSON.parse(JSON.stringify(textDto))
                                oldtextDto = oldtextDto.concat(Array(dynamicMaterialDTos.dynamicGroup.length - oldtextDto.length).fill({}))
                                form.setFieldsValue({ textDto: oldtextDto })
                            }
                        }}
                    />
                </Form.Item>
            </Card>

            <Form.List name="textDto">
                {(fields) => (<>
                    {fields.map(({ key, name, ...restField }, num) => (
                        <Card
                            title={type === 1 ? <strong style={{ fontSize: 14 }}>创意组{num + 1}</strong> : null}
                            className="cardResetCss"
                            style={{ marginTop: 10 }}
                            key={key}
                        >
                            <Space style={{ width: '100%' }} direction="vertical" size={10}>
                                {textList.map(item => {
                                    return <Form.Item
                                        {...restField}
                                        label={<strong>{item.label}</strong>}
                                        name={[name, item.value]}
                                        key={key}
                                        style={{ marginBottom: 0 }}
                                        rules={[{
                                            required: item.required, message: '请输入正确的' + item.label, validator: (rule, value) => {
                                                if (!value) {
                                                    return Promise.reject()
                                                } else if (!value.match(RegExp(item.restriction.textRestriction.textPattern))) {
                                                    return Promise.reject()
                                                } else if (txtLength(value) > item.restriction.textRestriction.maxLength) {
                                                    return Promise.reject()
                                                }
                                                return Promise.resolve()
                                            }
                                        }]}
                                    >
                                        <TextAideInput placeholder={'请输入' + item.label} style={{ width: 450 }} maxTextLength={item.restriction.textRestriction.maxLength} />
                                    </Form.Item>
                                })}
                            </Space>
                        </Card>
                    ))}
                </>)}
            </Form.List>
            <Form.Item className="submit_pull">
                <Space>
                    <Button onClick={onClose}>取消</Button>
                    <Button type="primary" htmlType="submit" className="modalResetCss">
                        确定
                    </Button>
                </Space>
            </Form.Item>
        </Form>
    </Modal>
}

export default React.memo(NewText)